// Sylvain MARECHAL
// sylvain.marechal1@libertysurf.fr
// Rev. 01/07/2005
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Globalization;
using System.Threading;

namespace tetris
{
    /// <summary>
    /// Classe de la forme principale
    /// </summary>
    public partial class FormTetris : Form//, ITetrisInfos
    {
        private string m_Langue = "fr";
        private int m_NiveauActuel = 0;

        /// <summary>
        /// Constructeur : On lit dans les "settings" de l'utilisateur la culture
        /// (ou langue) et on initialise l'interface avec ce type de culture
        /// </summary>
        public FormTetris()
        {
            // Initialise la culture a utliser
            m_Langue = Properties.Settings.Default.Langue;
            try
            {
                Thread.CurrentThread.CurrentUICulture = new CultureInfo(m_Langue);
            }
            catch (ArgumentException /*ae*/ )
            {
            }
            InitializeComponent();

            // Ajoute des "abonnes" afin de recuperer les informations
            // sur l'etat du jeu
            //tetrisPlayControl.TetrisInfos = this;
            tetrisPlayControl.TetrisInfos += new TetrisInfosEventHandler(TetrisInfosHandler);
            tetrisPlayControl.TetrisPerdu += new TetrisPerduEventHandler(TetrisPerduHandler);
        }

        /// <summary>
        /// Lance une nouvelle partie
        /// </summary>
        /// <param name="sender">Pas utilise</param>
        /// <param name="e">Pas utilise</param>
        private void btPlay_Click(object sender, EventArgs e)
        {
            tetrisPlayControl.Play();
            Refresh();
            tetrisPlayControl.Focus();
        }

        /// <summary>
        /// Met en pause la partie courante
        /// </summary>
        /// <param name="sender">Pas utilise</param>
        /// <param name="e">Pas utilise</param>
        private void btPause_Click(object sender, EventArgs e)
        {
            tetrisPlayControl.Pause();
            tetrisPlayControl.Focus();
        }

        /// <summary>
        /// Affiche les meilleurs scores
        /// </summary>
        /// <param name="sender">Pas utilise</param>
        /// <param name="e">Pas utilise</param>
        private void btScores_Click(object sender, EventArgs e)
        {
            FormShowScores fs = new FormShowScores();
            fs.ShowDialog(this);
            tetrisPlayControl.Focus();
        }

        /// <summary>
        /// Delegue appele par le controle TetrisPlayAreaControl 
        /// lorsque le score ou le niveau est succeptible d'avoir 
        /// change
        /// </summary>
        /// <param name="Sender">objet TetrisPlayAreaControl</param>
        /// <param name="e">argument TetrisInfosEventArgs</param>
        private void TetrisInfosHandler(object Sender, EventArgs e)
        {
            TetrisInfosEventArgs tieva = (TetrisInfosEventArgs)e;
            AfficheInfos(tieva.Score, tieva.Niveau, tieva.ProchainePiece);
        }

        /// <summary>
        /// -Met a jour les infos dans la zone prevue a cet effet
        /// -Change la couleur de fond du tableau si necessaire
        /// </summary>
        /// <param name="score">Le score</param>
        /// <param name="niveau">Le niveau</param>
        /// <param name="prochainePiece">La prochaine piece</param>
        private void AfficheInfos(int score, int niveau, TetrisCore.TetrisValeur prochainePiece)
        {
            tetrisApercuControl.PieceCourante = prochainePiece;
            tetrisApercuControl.Invalidate();
            lblNiveau.Text = ResourceLoader.GetString("Niveau_2p") + niveau;
            lblScore.Text = ResourceLoader.GetString("Score_2p")  + score;
            if (m_NiveauActuel != niveau)
            {
                m_NiveauActuel = niveau;
                if (niveau >= 0 && niveau < 8)
                {
                    BackgroundImage = 
                        new Bitmap(this.GetType(), "niveau" + m_NiveauActuel + ".bmp");
                }
            }
        }

        /// <summary>
        /// Delegue appele par le controle TetrisPlayAreaControl 
        /// quand la partie est perdue 
        /// change
        /// </summary>
        /// <param name="Sender">objet TetrisPlayAreaControl</param>
        /// <param name="e">argument TetrisPerduEventHandler</param>
        private void TetrisPerduHandler(object Sender, EventArgs e)
        {
            TetrisPerduEventArgs tpeva = (TetrisPerduEventArgs)e;
            Perdu(tpeva.Score);
        }
        
        /// <summary>
        /// Partie perdue
        /// </summary>
        /// <param name="score"></param>
        private void Perdu(int score)
        {
            CScores css = new CScores();
            if (css.IsBest(score))
            {
                FormAddScore fas = new FormAddScore();
                DialogResult res = fas.ShowDialog(this);
                if( res == DialogResult.OK )
                {
                    string nom = fas.Nom;
                    css.Add(nom, score, DateTime.Now);
                }
            }
        }

        /// <summary>
        /// Click sur quitter
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btExit(object sender, EventArgs e)
        {
            Application.Exit();
        }

        /// <summary>
        /// Affiche une boite a propos
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btAPropos(object sender, EventArgs e)
        {
            //toolStripStatusLabel.Text = ResourceLoader.GetString("APropos");
            FormAPropos fap = new FormAPropos();
            fap.ShowDialog();
        }

        /// <summary>
        /// Click sur menu langue
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void langueToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FormLang fm = new FormLang(m_Langue);
            DialogResult res = fm.ShowDialog(this);
            if( res == DialogResult.OK && fm.Langue != "" )
            {
                Properties.Settings.Default.Langue = fm.Langue;
                Properties.Settings.Default.Save(); 
            }
        }
    }
}