// Sylvain MARECHAL
// sylvain.marechal1@libertysurf.fr
// Rev. 01/07/2005
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace tetris
{
    /// <summary>
    /// Utilisation de la classe TetrisControl afin d'afficher
    /// l'apercu de la prochaine piece
    /// </summary>
    class TetrisApercuAreaControl : TetrisControl
    {
        private TetrisCore.TetrisValeur[,] m_tabApercu;

        /// <summary>
        /// Le constructeur initialise un table de 6 lignes, 5 colonnes
        /// qui servira a afficher la prochaine piece a afficher
        /// </summary>
        public TetrisApercuAreaControl()
        {
            Lignes = 6;
            Colonnes = 5;
            m_tabApercu = new TetrisCore.TetrisValeur[Lignes, Colonnes];
            initTabApercu(TetrisCore.TetrisValeur.TETRIS_RIEN);
        }

        /// <summary>
        /// Initialise le tableau en centrant dedans la prochaine piece
        /// a afficher
        /// </summary>
        /// <param name="couleur"></param>
        private void initTabApercu(TetrisCore.TetrisValeur couleur)
        {
            for (int lig = 0; lig < Lignes; lig++)
                for (int col = 0; col < Colonnes; col++)
                    m_tabApercu[lig, col] = TetrisCore.TetrisValeur.TETRIS_RIEN;
            switch (couleur)
            {
                case TetrisCore.TetrisValeur.CARRE_BLEU:
                    m_tabApercu[2, 2] = couleur;
                    m_tabApercu[3, 2] = couleur;
                    m_tabApercu[2, 3] = couleur;
                    m_tabApercu[3, 3] = couleur;
                    break;
                case TetrisCore.TetrisValeur.LIGNE_ROUGE:
                    m_tabApercu[1, 2] = couleur;
                    m_tabApercu[2, 2] = couleur;
                    m_tabApercu[3, 2] = couleur;
                    m_tabApercu[4, 2] = couleur;
                    break;
                case TetrisCore.TetrisValeur.TRIANGLE_VERT:
                    m_tabApercu[2, 2] = couleur;
                    m_tabApercu[3, 1] = couleur;
                    m_tabApercu[3, 2] = couleur;
                    m_tabApercu[3, 3] = couleur;
                    break;
                case TetrisCore.TetrisValeur.ZIG1_CYAN:
                    m_tabApercu[2, 1] = couleur;
                    m_tabApercu[2, 2] = couleur;
                    m_tabApercu[3, 2] = couleur;
                    m_tabApercu[3, 3] = couleur;
                    break;
                case TetrisCore.TetrisValeur.ZIG2_GRIS:
                    m_tabApercu[2, 2] = couleur;
                    m_tabApercu[2, 3] = couleur;
                    m_tabApercu[3, 1] = couleur;
                    m_tabApercu[3, 2] = couleur;
                    break;
                case TetrisCore.TetrisValeur.L1_ROSE:
                    m_tabApercu[2, 1] = couleur;
                    m_tabApercu[2, 2] = couleur;
                    m_tabApercu[2, 3] = couleur;
                    m_tabApercu[3, 3] = couleur;
                    break;
                case TetrisCore.TetrisValeur.L2_JAUNE:
                    m_tabApercu[2, 1] = couleur;
                    m_tabApercu[2, 2] = couleur;
                    m_tabApercu[2, 3] = couleur;
                    m_tabApercu[3, 1] = couleur;
                    break;
                default:
                    break;
            }

        }

        /// <summary>
        /// Permet de mettre a jour la prochaine peice
        /// </summary>
        public TetrisCore.TetrisValeur PieceCourante
        {
            set { initTabApercu(value); } 
        }


        /// <summary>
        /// Renvoie la couleur de la case (iLig, jCol). Cette methode
        /// est utilisee par l'appelant pour redessiner le controle
        /// </summary>
        /// <param name="iLig">Ligne</param>
        /// <param name="jCol">Colonne</param>
        /// <returns></returns>
        protected override TetrisCore.TetrisValeur getCouleur(int iLig, int jCol)
        {
            return m_tabApercu[iLig, jCol];
        }

    }
}
