// Sylvain MARECHAL
// sylvain.marechal1@libertysurf.fr
// Rev. 01/07/2005
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;


namespace tetris
{
    /// <summary>
    /// Classe de base des 2 controles tetris. (TetrisApercu et TetrisPlay)
    /// Ce controle sait afficher un carre de couleur sur un fond noir.
    /// Toute l'intelligence est implementee dans les classes filles
    /// </summary>
    public partial class TetrisControl : UserControl
    {
        private int m_xCol = 10;
        private int m_yLig = 20;
        private int m_xMorceau = 0;
        private int m_yMorceau = 0;
        private int m_xGauche = 0;
        private int m_yHaut = 0;

        /// <summary>
        /// Nombre de colonnes
        /// </summary>
        public int Colonnes
        {
            get { return m_xCol; }
            set { m_xCol = value; }
        }

        /// <summary>
        /// Nombre de lignes
        /// </summary>
        public int Lignes
        {
            get { return m_yLig; }
            set { m_yLig = value; }
        }

        /// <summary>
        /// Conversion d'un code TetrisCore.TetrisValeur en couleur
        /// </summary>
        /// <param name="couleur">Code de type de piece</param>
        /// <returns>La couleur correspondante</returns>
        static public Color TetrisValeur2Color(TetrisCore.TetrisValeur couleur)
        {
            Color cr;
            switch (couleur)
            {
                case TetrisCore.TetrisValeur.CARRE_BLEU: cr = Color.Blue; break;
                case TetrisCore.TetrisValeur.LIGNE_ROUGE: cr = Color.Red; break;
                case TetrisCore.TetrisValeur.TRIANGLE_VERT: cr = Color.Green; break;
                case TetrisCore.TetrisValeur.ZIG1_CYAN: cr = Color.Cyan; break;
                case TetrisCore.TetrisValeur.ZIG2_GRIS: cr = Color.Gray; break;
                case TetrisCore.TetrisValeur.L1_ROSE: cr = Color.Pink; break;
                case TetrisCore.TetrisValeur.L2_JAUNE: cr = Color.Yellow; break;
                default: cr = Color.Black; break;
            }
            return cr;
        }

        /// <summary>
        /// Affichage d'un carre de la couleur demandee
        /// </summary>
        /// <param name="g"></param>
        /// <param name="yLig"></param>
        /// <param name="xCol"></param>
        /// <param name="couleur"></param>
        public void AfficheMorceauPiece(System.Drawing.Graphics g,
            int yLig, int xCol, TetrisCore.TetrisValeur couleur)
        {
            Color cr = TetrisValeur2Color( couleur );
            System.Drawing.SolidBrush sBrush = 
                new System.Drawing.SolidBrush(cr);
            g.FillRectangle(sBrush, 
                    m_xGauche + (xCol) * (m_xMorceau + 1),
                    m_yHaut + (yLig) * (m_yMorceau + 1), 
                    m_xMorceau, m_yMorceau);
            sBrush.Dispose();
        }

        /// <summary>
        /// L'equivalent WM_PAINT
        /// </summary>
        /// <param name="pe"></param>
        protected override void OnPaint(PaintEventArgs pe)
        {
            // Call the OnPaint method of the base class.
            base.OnPaint(pe);

            m_xMorceau = ( this.Size.Width - m_xCol + 1)/ m_xCol;
            m_xGauche = (this.Size.Width - m_xCol * (m_xMorceau + 1)) / 2;
            m_yMorceau = (this.Size.Height - m_yLig + 1) / m_yLig;
            m_yHaut = (this.Size.Height - m_yLig * (m_yMorceau + 1)) / 2;

            for (int iLig = 0; iLig < Lignes; iLig++)
            {
                for (int jCol = 0; jCol < Colonnes; jCol++)
                {
                    TetrisCore.TetrisValeur couleur = getCouleur(iLig, jCol);
                    AfficheMorceauPiece(pe.Graphics, iLig, jCol, couleur);
                }
            }
        }
        /// <summary>
        /// Methode qui doit etre surchagee par la classe fille
        /// afin de fournir le code de la case a afficher
        /// </summary>
        /// <param name="iLig">Ligne</param>
        /// <param name="jCol">Colonne</param>
        /// <returns>Le code couleur</returns>
        protected virtual TetrisCore.TetrisValeur getCouleur(int iLig, int jCol)
        {
            // On renvoie du rouge par ex ... (Cette methode doit etre surchargee)
            return TetrisCore.TetrisValeur.LIGNE_ROUGE;
        }
    }


}
