// Sylvain MARECHAL
// sylvain.marechal1@libertysurf.fr
// Rev. 01/07/2005
using System;


namespace tetris
{
    /// <summary>
    /// Le coeur de tetris
    /// On doit fournir un objet ITetrisGraphics afin de faire l'affichage.
    /// Il aurait ete possible d'utiliser des evenements
    /// </summary>
    public class TetrisCore : ITetrisGraphics 
    {
        
        /// <summary>
        /// Cette enum  decrit les types de pieces et leur couleur
        /// </summary>
        public enum TetrisValeur
        {
            /// <summary>
            /// CARRE_BLEU
            /// </summary>
            CARRE_BLEU = 0,
            /// <summary>
            /// LIGNE_ROUGE 
            /// </summary>
            LIGNE_ROUGE = 1,
            /// <summary>
            /// TRIANGLE_VERT
            /// </summary>
            TRIANGLE_VERT = 2,
            /// <summary>
            /// ZIG1_CYAN
            /// </summary>
            ZIG1_CYAN = 3,
            /// <summary>
            /// ZIG2_GRIS
            /// </summary>
            ZIG2_GRIS = 4,
            /// <summary>
            /// L1_ROSE
            /// </summary>
            L1_ROSE = 5,
            /// <summary>
            /// L2_JAUNE
            /// </summary>
            L2_JAUNE = 6,
            /// <summary>
            /// TETRIS_RIEN doit etre la plus grande valeur 
            /// (utilise ds random)
            /// </summary>
            TETRIS_RIEN = 7
        }

        /// <summary>
        /// Declaration des pieces : 
	    /// On utilise  cet effet un tableau  4 dimensions
	    /// -Le premier indice correspond au type de piece
	    /// -Le 2 indice correspond  la rotation en cours
	    /// -Le 3 et 4 indices correspondent aux coordonnees 
	    ///  relatives de chaque element composant une piece
	    ///  Ces coordonnees sont dans un format {ligne, colonne}
        ///
	    /// On remarque par ailleurs que l'ordre des elements
	    /// du tableau correspond  l'ordre de l'enumeration
	    /// TetrisValeur, cad CARRE, puis LIGNE, puis TRIANGLE etc.
        /// </summary>
        static int [,,,] cs_aPieces//[/*7*/][/*4*/][/*4*/][/*2*/]
            =
        {
            {	// Le carre (toujours ds la meme position)
    	    {
    		    {0,0}, {0,1},
    		    {1,0}, {1,1}
    	    },
    	    {
    		    {0,0}, {0,1},
    		    {1,0}, {1,1} 
    	    },
    	    {
    		    {0,0}, {0,1},
    		    {1,0}, {1,1} 
    	    },
    	    {
    		    {0,0}, {0,1},
    		    {1,0}, {1,1} 
    	    }
            },

            {	// La ligne
    	    {
    		    {0,-1},{0,0},{0,1},{0,2}
    	    },
    	    {
    		    {-1,0},
    		    {0,0},
    		    {1,0},
    		    {2,0}
    	    },
    	    {
    		    {0,-1},{0,0},{0,1},{0,2}
    	    },
    	    {
    		    {-1,0},
    		    {0,0},
    		    {1,0},
    		    {2,0}
    	    }
            },

            {	// Le triangle
    	    {
    		        {-1,0},
    	        {0,-1}, {0,0}, {0,1} 
    	    },
    	    {
    		        {-1,0},
    		        {0,0}, {0,1},
    		        {1,0}
    	    },
    	    {
    	        {0,-1}, {0,0}, {0,1},
    		        {1,0},
    	    },
    	    {
    		        {-1,0},
    	        {0,-1}, {0,0},
    		        {1,0}
    	    }
            },

            {	// Le zigzag 1
    	    {
    	        {0,-1}, {0,0},
    		        {1,0}, {1,1} 
    	    },
    	    {
    		        {-1,0},
    	        {0,-1}, {0,0} ,
    	        {1,-1}
    	    },
    	    {
    	        {0,-1}, {0,0},
    		        {1,0}, {1,1} 
    	    },
    	    {
    		        {-1,0},
    	        {0,-1}, {0,0} ,
    	        {1,-1}
    	    }
            },

            {	// Le zigzag 2
    	    {
    		        {0,0}, {0,1},
    	        {1,-1}, {1,0} 
    	    },
    	    {
    	        {-1,-1},
    	        {0,-1}, {0,0}, 
    		        {1,0} 
    	    },
    	    {
    		        {0,0}, {0,1},
    	        {1,-1}, {1,0} 
    	    },
    	    {
    	        {-1,-1},
    	        {0,-1}, {0,0}, 
    		        {1,0} 
    	    }
            },
            
            {	// Le L 1
    	    {
    	        {0,-1}, {0,0},   {0,1},
    			        {1,1} 
    	    },
    	    {
    		        {-1,0},
    		        {0,0},
    	        {1,-1}, {1,0}
    	    },
    	    {
    	        {-1,-1},
    	        {0,-1}, {0,0},   {0,1},
    	    },
    	    {
    		        {-1,0}, {-1,1},
    		        {0,0},
    		        {1,0}
    	    }
            },

            {	// Le L 2
    	    {
    	        {0,-1}, {0,0}, {0,1},
    	        {1,-1} 
    	    },
    	    {
    	        {-1,-1},{-1,0},
    		        {0,0},
    		        {1,0}
    	    },
    	    {
    			    {-1,1},
    	        {0,-1}, {0,0}, {0,1},
    	    },
    	    {
    		        {-1,0},
    		        {0,0},
    		        {1,0}, {1,1}
    	    }
            }
        };
        
        /// <summary>
        /// Dimension du tableau 
        /// </summary>
        public static readonly int TETRIS_COLONNES = 10;

        /// <summary>
        /// Dimension du tableau 
        /// </summary>
        public static readonly int TETRIS_LIGNES = 20;
        private int cs_Colonnes = TETRIS_COLONNES;
        private int cs_Lignes = TETRIS_LIGNES;

        // Declaration du tableau
        private TetrisValeur[,] m_aTetris = new TetrisValeur[TETRIS_LIGNES, TETRIS_COLONNES];
        private int m_colTetris;

	    // Delai entre 2 mouvements : Ce delai va diminuer en meme
	    // temps que le niveau du jeu augmente
        private int m_Delai;
        private bool m_fAfficheProchainePiece = true;

	    // Definit la position de la piece dans le tableau
        private int m_ligTetris;
        private int m_Niveau;

	    // Definit la piece qui est actuellement affichee et
	    // en cours de descente
        private TetrisValeur m_PieceCourante;

        // Voir ChoisirPiece() 
        private bool m_PremierChoix = true;

	    // Definit la prochaine piece qui va s'afficher. Celle
	    // ci est utilisee afin de la montrer  l'utilisateur
	    // dans un coin de l'ecran
	    // Par defaut, on active l'apercu
        private TetrisValeur m_ProchainePiece;

	    // Definit la rotation en cours
        private int m_Rotation;

        // Score et niveau
        private int m_Score;
        
        private ITetrisGraphics m_TetrisGraphics;
        
        /// <summary>
        /// Implementation de la classe ITetrisGraphics
        /// </summary>
        /// <param name="score">Score actuel</param>
        /// <param name="niveau">Score actuel</param>
        /// <param name="prochainePiece">La prochaien peice</param>
        /// <see cref="ITetrisGraphics"/>
        public void AfficheInfos(int score, int niveau, TetrisValeur prochainePiece)
        {
            m_TetrisGraphics.AfficheInfos( score, niveau, prochainePiece);
        }

        /// <summary>
        /// Implementation de la classe ITetrisGraphics
        /// </summary>
        /// <param name="ligne">la ligne</param>
        /// <param name="colonne">la colonne</param>
        /// <param name="couleur">type de piece</param>
        /// <see cref="ITetrisGraphics"/>
        public void AfficheMorceauPiece(int ligne, int colonne, TetrisValeur couleur)
        {
            m_TetrisGraphics.AfficheMorceauPiece(ligne, colonne, couleur);
        }

        /// <summary>
        /// Implementation de la classe ITetrisGraphics
        /// </summary>
        /// <see cref="ITetrisGraphics"/>
        public void AfficheTetris()
        {
            m_TetrisGraphics.AfficheTetris();
        }

        /// <summary>
        /// Affiche la piece  la position indiquee par les variables
        /// m_Rotation, m_ligTetris et m_colTetris.
	    /// Appelle la fonction AfficheMorceauPiece() du module d'affichage
	    /// qui realise l'affichage.
	    /// On note que si la valeur de couleur est TETRIS_RIEN, alors la piece est
	    /// effacee
        /// </summary>
        /// <param name="couleur">type de piece</param>
        private void AffichePiece(TetrisValeur couleur)
        {
            for( int morceau = 0; morceau < 4; morceau ++ )
            {
                int PieceCourante = (int)m_PieceCourante;
	    	    int ligneRel = 
	    	        cs_aPieces[PieceCourante,m_Rotation%4,morceau,0];
	    	    int colonneRel = 
	    	        cs_aPieces[PieceCourante,m_Rotation%4,morceau,1];
    	
	    	    int ligne = m_ligTetris + ligneRel;
	    	    int colonne = m_colTetris + colonneRel;
    	
	    	    if( ligne >= 0 && ligne < cs_Lignes &&
	    	        colonne >= 0 && colonne < cs_Colonnes )
	    	    {
	    	        // On change la couleur de cette cellule
	    	        m_aTetris[ligne,colonne] = couleur;
    	    	    
	    	        // On appelle la fonction qui est chargee d'afficher la cellule
	    	        AfficheMorceauPiece( ligne, colonne, couleur );
	    	    }
            }
        }

        /// <summary>
        /// Modifie le score et le niveau en fonction du nombre de lignes
        /// realisees. Agit sur les variables
        /// </summary>
        /// <param name="nombreNouvellesLignes">Le nombre de lignes completes</param>
        private void ChangeScoreEtNiveau( int nombreNouvellesLignes )
        {
            m_Score += nombreNouvellesLignes;
            if( m_Score > 100 )
            {
	    	    m_Niveau = 7;
	    	    m_Delai = 100;
            }
            else if( m_Score > 70 )
            {
	    	    m_Niveau = 6;
	    	    m_Delai = 200;
            }
            else if( m_Score > 50 )
            {
	    	    m_Niveau = 5;
	    	    m_Delai = 300;
            }
            else if( m_Score > 40 )
            {
	    	    m_Niveau = 4;
	    	    m_Delai = 400;
            }
            else if( m_Score > 30 )
            {
	    	    m_Niveau = 3;
	    	    m_Delai = 500;
            }
            else if( m_Score > 20 )
            {
	    	    m_Niveau = 2;
	    	    m_Delai = 600;
            }
            else if( m_Score > 10 )
            {
	    	    m_Niveau = 1;
	    	    m_Delai = 800;
            }
        }

        /// <summary>
        /// Choisit la piece qui doit s'afficher ainsi que la prochaine
        /// piece  venir en utilisant PieceAuHasard().
        /// Initialise les variables m_PieceCourante et m_ProchainePiece
        /// </summary>
        private void ChoisirPiece()
        {
            // Cette fonction est appelee au debut du jeu puis chaque fois qu'une
            // piece arrive en bas.
            // Elle initialise les variables m_ProchainePiece et m_PieceCourante.
            // Elle utilise PieceAuHasard()
            // 
            // La premiere fois (variable m_PremierChoix), 
            // cette fonction initialise les variables m_ProchainePiece et m_PieceCourante.
            // 
            // Les fois suivantes, elle initialise m_PieceCourante avec m_ProchainePiece, 
            // puis m_ProchainePiece au hasard.

            if( m_PremierChoix )
            {
	    	    m_PieceCourante = PieceAuHasard();
	    	    m_PremierChoix = false;
            }
            else
            {
	    	    m_PieceCourante = m_ProchainePiece;
            }
            m_ProchainePiece = PieceAuHasard();
        }

        /// <summary>
        /// Decale les lignes du tableau d'index inferieur  iLigne.
        /// Cette fonction est appelee lorsque des lignes sont completes
        /// </summary>
        /// <param name="iLigne">La ligne a partir d'ou se fait le decalage</param>
        private void DecaleTableau(int iLigne)
        {
            for( int i = iLigne; i > 0; i -- )
            {
	    	    for( int j = 0; j < cs_Colonnes; j ++ )
	    	    {
	    	        m_aTetris[i,j] = m_aTetris[i-1,j];
	    	    }
            }
        }

        /// <summary>
	    /// alors on selectionne une nouvelle piece qu'on essaye d'afficher.
	    /// Si cela n'est pas possible, alors cela signifie que la partie est
	    /// perdue.
 	    /// Utilise la fonction GereMouvementPiece() qui fait tout le travail
        /// </summary>
        /// <returns>Renvoie false si la partie est perdu, true sinon</returns>
        public bool Descend()
        {
            bool ret;

            ret = GereMouvementPiece( 1, 0, 0 );
            if( ret == false )
            {
	    	    PieceEnBas();
	    	    NouvellePiece();
                AfficheInfos(this.Score, this.Niveau, this.ProchainePiece);
	    	    ret = GereMouvementPiece( 1, 0, 0 );;
            }
            return ret;
        }

        /// <summary>
        /// Essaye de se deplacer  droite
        /// </summary>
        public void Droite()
        {
            GereMouvementPiece( 0, 1, 0 );
        }

        /// <summary>
        /// Essaye de se deplacer  gauche
        /// </summary>
        public void Gauche()
        {
            GereMouvementPiece( 0, -1, 0 );
        }

        /// <summary>
        /// Essaie de deplacer la piece  une nouvelle position.
	    /// * si ligIncrement = 1 => On descend
	    /// * si colIncrement = 1, on va  droite
	    /// * si colIncrement = -1, on va  gauche
	    /// * si rotIncrement = 1, on tourne
        /// </summary>
        /// <param name="ligIncrement">0 ou 1</param>
        /// <param name="colIncrement">-1, 0 ou 1</param>
        /// <param name="rotIncrement">0 ou 1</param>
        /// <returns></returns>
        private bool GereMouvementPiece( int ligIncrement, int colIncrement, int rotIncrement )
        {
            bool ret = false;

            // On commence par effacer
            AffichePiece( TetrisValeur.TETRIS_RIEN );
            
            // On teste si on peut bouger
            if( PiecePeutBouger( ligIncrement, colIncrement, rotIncrement ) )
            {
	    	    // On change les variables
	    	    PieceBouge( ligIncrement, colIncrement, rotIncrement );
    	
	    	    // On a bouge
	    	    ret = true;
            }
            
            // On reaffiche
            AffichePiece( m_PieceCourante );

            return ret;
        }

        /// <summary>
        /// Renvoie le type de piece
        /// </summary>
        /// <param name="ligne">Ligne</param>
        /// <param name="colonne">Colonne</param>
        /// <returns>Type de piece</returns>
        public TetrisValeur getCouleur(int ligne, int colonne)
        {
            // Renvoie la couleur de la piece (m_aTetris[ligne][colonne])
            return (TetrisValeur)m_aTetris[ligne, colonne];
        }

        /// <summary>
        /// Le delai en millisecondes
        /// </summary>
        public int Delai
        {
            get { return m_Delai; }
            set { m_Delai = value; }
        }

        /// <summary>
        /// Le niveau de 0 a 8
        /// </summary>
        public int Niveau
        {
            get { return m_Niveau; }
        }

        /// <summary>
        /// Le type de piece courante
        /// </summary>
        public TetrisValeur PieceCourante
        {
            get { return m_PieceCourante; }
        }

        /// <summary>
        /// Le type de la prochaine piece
        /// </summary>
        public TetrisValeur ProchainePiece
        {
            get
            {
                if( m_fAfficheProchainePiece ) return m_ProchainePiece;
                return TetrisValeur.TETRIS_RIEN; 
            }
        }

        /// <summary>
        /// Lescore actuel
        /// </summary>
        public int Score 
        {
            get { return m_Score; }
        }

        /// <summary>
        /// Verifie si la ligne iLigne est complete
        /// </summary>
        /// <param name="iLigne">Numero de ligne a tester</param>
        /// <returns>true si la ligne est complete</returns>
        private bool LigneComplete( int iLigne )
        {
            for( int j = 0; j < cs_Colonnes; j ++ )
            {
	    	    if( m_aTetris[iLigne,j] == TetrisValeur.TETRIS_RIEN )
	    	    {
	    	        return false; // La ligne n'est pas complete
	    	    }
            }        
            return true; // Ligne complete
        }

        /// <summary>
        /// Initialise les variables pour qu'une nouvelle partie 
        /// puisse commencer
        /// </summary>
        private void NouvellePartie()
        {
            // Tableau initialise  TETRIS_RIEN
            for( int i = 0; i < cs_Lignes; i ++ )
            {
	    	    for( int j = 0; j < cs_Colonnes; j ++ )
	    	    {
	    	        m_aTetris[i,j] = TetrisValeur.TETRIS_RIEN;
	    	    }
            }

            // Variables m_Delai initialisee  1000, m_Score  0
            // m_Niveau  0
            m_Delai = 1000;
            m_Score = 0;
            m_Niveau = 0;
        }

        /// <summary>
        /// Selectionne la nouvelle piece  utiliser en appelant 
        /// la fonction ChoisirPiece(). 
        /// Reinitialise les variables de position
        /// </summary>
        public void NouvellePiece()
        {
            // Appelle ChoisirPiece() afin d'initialiser m_PieceCourante et 
            // m_ProchainePiece
            // Reinitialise m_ligTetris  0, m_colTetris  cs_Colonnes/2 et 
            // m_Rotation  0 afin que la nouvelle piece s'affiche en haut 
            // et au milieu du tableau
            ChoisirPiece();
            m_ligTetris = 0;
            m_colTetris = cs_Colonnes/2;
            m_Rotation = 0;
        }

        private Random m_Random = new Random();

        /// <summary>
        /// Tire une piece au hasard
        /// </summary>
        /// <returns>Le type de piece</returns>
        private TetrisValeur PieceAuHasard()
        {
            int maxVal = (int)TetrisValeur.TETRIS_RIEN;
            int val = m_Random.Next(maxVal);
            return (TetrisValeur)val;
        }

        /// <summary>
        /// Incremente les variables de position
        /// </summary>
        /// <param name="ligIncrement"></param>
        /// <param name="colIncrement"></param>
        /// <param name="rotIncrement"></param>
        private void PieceBouge( int ligIncrement, int colIncrement, int rotIncrement )
        {
            // Incremente m_Rotation, m_ligTetris et m_colTetris avec 
            // ligIncrement, colIncrement et rotIncrement
            m_Rotation += rotIncrement;
            m_ligTetris += ligIncrement;
            m_colTetris += colIncrement;
        }

        /// <summary>
	    /// Verifie la presence de lignes completes et les efface le
	    /// cas echeant puis decale toutes celles qui sont au dessus.
	    /// Utilise les fonctions LigneComplete() et DecaleTableau().
	    /// Si des decalages ont lieu, le tableau est redessine
	    /// Appelle la fonction AfficheTetris() du module d'affichage
	    /// qui realise l'affichage
	    /// Le cas echeant, elle incremente le score en appelant
        /// la fonction ChangeScoreEtNiveau()
        /// </summary>
        private void PieceEnBas()
        {
            int nombreNouvellesLignes = 0;
            for( int i = cs_Lignes-1; i >= 0; i -- )
            {
	    	    if( LigneComplete(i) )
	    	    {
	    	        DecaleTableau(i);
	    	        nombreNouvellesLignes ++;
	    	        i ++;
	    	    }
            }
            if( nombreNouvellesLignes != 0 )
            {
	    	    AfficheTetris();
	    	    ChangeScoreEtNiveau( nombreNouvellesLignes );
            }
        }

        /// <summary>
        /// Renvoie 1 si la piece peut bouger dans une des direction indiquee par
        /// les arguments.
        /// </summary>
        /// <param name="ligIncrement"></param>
        /// <param name="colIncrement"></param>
        /// <param name="rotIncrement"></param>
        /// <returns>true si la piece peut bouger</returns>
        private bool PiecePeutBouger( int ligIncrement, int colIncrement, int rotIncrement )
        {
            // Pour chaque morceau composant la piece, on calcule la
            // prochaine position, ensuite on verifie que si cette
            // position est dans le tableau, elle est libre
            for( int morceau = 0; morceau < 4; morceau ++ )
            {
	    	    // Calcul de la position
	    	    // On tient compte de la rotation
	    	    int ligneRel = cs_aPieces[(int)m_PieceCourante,(m_Rotation+rotIncrement)%4,morceau,0];
	    	    int colonneRel = cs_aPieces[(int)m_PieceCourante,(m_Rotation+rotIncrement)%4,morceau,1];
    	
	    	    // On ajuste la position en fonction des translations (droite/gauche/bas)
	    	    int ligne = m_ligTetris + ligneRel + ligIncrement;
	    	    int colonne = m_colTetris + colonneRel + colIncrement;
    	
	    	    // Est-ce que cette position est dans le tableau ?
	    	    if( ligne >= 0 && ligne < cs_Lignes &&
	    	        colonne >= 0 && colonne < cs_Colonnes )
	    	    {
	    	        // On est dans le tableau
	    	        if( m_aTetris[ligne,colonne] != TetrisValeur.TETRIS_RIEN )
	    	        {
	    	            return false; // La cellule est occupee
	    	        }
	    	    }
	    	    else
	    	    {
	    	        return false; // On est sur un bord
	    	    }
            }

            // Cette position est libre
            return true;
        }
        
        /// <summary>
        /// Apercu visible ou pas
        /// </summary>
        public bool Apercu
        {
            get { return m_fAfficheProchainePiece; }
            set { m_fAfficheProchainePiece = value; }
        }
        
        /// <summary>
        /// Creation d'un TetrisCore
        /// </summary>
        /// <param name="tg">Un objet derivant de ITetrisGraphics</param>
        public TetrisCore( ITetrisGraphics tg )
        {
            m_TetrisGraphics = tg;
            NouvellePartie();
            NouvellePiece();
        }

        /// <summary>
	    /// Descend la piece jusqu'en bas, puis selectionne une nouvelle piece.
        /// Si on ne peut pas afficher la nouvelle piece, cela signifie que la
        /// partie est perdue, et la fonction renvoie false.
        /// Utilise la fonction GereMouvementPiece() qui fait tout le travail
        /// </summary>
        /// <returns>Renvoie false si la partie est perdu, true sinon</returns>
        public bool Tombe()
        {
            bool ret;

            while( GereMouvementPiece( 1, 0, 0 ) );
            
            PieceEnBas();
            NouvellePiece();
            AfficheInfos( this.Score, this.Niveau, this.ProchainePiece );
            
            ret = GereMouvementPiece( 1, 0, 0 );;
            return ret;
        }

        /// <summary>
        /// Essaye de tourner.
        /// </summary>
        public void Tourne()
        {
            GereMouvementPiece( 0, 0, 1 );
        }
    }
}