// Sylvain MARECHAL
// sylvain.marechal1@libertysurf.fr
// Rev. 01/07/2005
using System;
using System.Collections.Generic;
using System.Text;

namespace tetris
{
    /// <summary>
    /// La signature du delegue a utiliser pour etre prevenu 
    /// de cet evenement
    /// </summary>
    /// <param name="Sender">L'objet qui lance l'evenement</param>
    /// <param name="e">L'evenement decrit ci dessous</param>
    public delegate void TetrisInfosEventHandler( object Sender, EventArgs e);

    /// <summary>
    /// Evenement pour mettre a jour l'interface avec
    /// le score le niveau et la prochaine piece
    /// </summary>
    class TetrisInfosEventArgs: EventArgs
    {
        private int m_Score;
        private int m_Niveau;
        private TetrisCore.TetrisValeur m_ProchainePiece;

        /// <summary>
        /// Renvoie le score
        /// </summary>
        public int Score
        { 
            get { return m_Score; }
        }

        /// <summary>
        /// Renvoie le niveau
        /// </summary>
        public int Niveau
        {
            get {return m_Niveau;}
        }

        /// <summary>
        /// Renvoie la prochaine piece
        /// </summary>
        public TetrisCore.TetrisValeur ProchainePiece
        {
            get { return m_ProchainePiece;}
        }

        /// <summary>
        /// Creation de l'evenement
        /// </summary>
        /// <param name="score">Le score</param>
        /// <param name="niveau">Le niveau</param>
        /// <param name="prochainePiece">La prochaine piece</param>
        public TetrisInfosEventArgs(int score, int niveau, TetrisCore.TetrisValeur prochainePiece)
        {
            m_Score = score;
            m_Niveau = niveau;
            m_ProchainePiece = prochainePiece;
        }
    }
}
