// Sylvain MARECHAL
// sylvain.marechal1@libertysurf.fr
// Rev. 01/07/2005
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace tetris
{
    /// <summary>
    /// Le composant tetris de jeu a inserer dans la form
    /// On doit lui mettre le focus pour qu'il capte les touches
    /// haut, bas, gauche, droite et espace.
    /// 
    /// </summary>
    class TetrisPlayAreaControl : TetrisControl, ITetrisGraphics
    {
        private TetrisCore m_TetrisCore = null;
        private bool m_Jeu = false;
        private bool m_Pause = false;
        private System.Windows.Forms.Timer m_Timer = null;
        
        //ITetrisInfos m_TetrisInfos = null;
        public event TetrisInfosEventHandler TetrisInfos;
        public event TetrisPerduEventHandler TetrisPerdu;

        /// <summary>
        /// Constructeur
        /// </summary>
        public TetrisPlayAreaControl()
        {
            Lignes = 20;
            Colonnes = 10;
            m_TetrisCore = new TetrisCore(this);
            m_Timer = new System.Windows.Forms.Timer();
            m_Timer.Tick += new EventHandler(this.TimerEventProcessor);
        }

        /// <summary>
        /// Renvoie la couleur pour la case iLig, jCol
        /// </summary>
        /// <param name="iLig">Ligne</param>
        /// <param name="jCol">Colonne</param>
        /// <returns>Le type de piece</returns>
        protected override TetrisCore.TetrisValeur getCouleur(int iLig, int jCol)
        {
            return m_TetrisCore.getCouleur(iLig, jCol);
        }

        /// <summary>
        /// Cpate l'appui sur les fleches et la touche espace
        /// </summary>
        /// <param name="keyData">Touche pressee</param>
        /// <returns>true</returns>
        protected override bool IsInputKey(Keys keyData)
        {
            if (m_Jeu && !m_Pause)
            {
                switch (keyData)
                {
                    case Keys.Space:
                        {
                            m_TetrisCore.Tombe();
                            break;
                        }
                    case Keys.Left:
                        {
                            m_TetrisCore.Gauche();
                            break;
                        }
                    case Keys.Right:
                        {
                            m_TetrisCore.Droite();
                            break;
                        }
                    case Keys.Up:
                        {
                            m_TetrisCore.Tourne();
                            break;
                        }
                    case Keys.Down:
                        {
                            m_TetrisCore.Descend();
                            break;
                        }
                }
            }
            return true;
        }


        #region ITetrisGraphics Members

        /// <summary>
        /// Indique que le score a change etc.
        /// </summary>
        /// <param name="score">Score actuel</param>
        /// <param name="niveau">Niveau actuel</param>
        /// <param name="prochainePiece">Prochaine piece</param>
        public void AfficheInfos(int score, int niveau, TetrisCore.TetrisValeur prochainePiece)
        {
            if (TetrisInfos != null)
            {
                TetrisInfosEventArgs tieva = new TetrisInfosEventArgs( score, niveau, prochainePiece );
                TetrisInfos(this, tieva);
            }
            /*
            if (m_TetrisInfos != null )
            {
                m_TetrisInfos.AfficheInfos( score, niveau, prochainePiece );
            }*/
        }

        /// <summary>
        /// Affiche la case ligne, colonne avec le type de piece couleur
        /// </summary>
        /// <param name="ligne">La ligne</param>
        /// <param name="colonne">La colonne</param>
        /// <param name="couleur">La couleur</param>
        public void AfficheMorceauPiece(int ligne, int colonne, TetrisCore.TetrisValeur couleur)
        {
            Graphics g = this.CreateGraphics();
            AfficheMorceauPiece( g, ligne, colonne, couleur );
        }

        /// <summary>
        /// Reaffiche tout
        /// </summary>
        public void AfficheTetris()
        {
            Invalidate();
        }

        #endregion

        /// <summary>
        /// Methode appelee par le timer afin de faire descendre
        /// la piece d'un cran
        /// </summary>
        /// <param name="obj"></param>
        /// <param name="myEventArgs"></param>
        private void TimerEventProcessor(Object obj, EventArgs myEventArgs)         
        {
            if (m_Jeu && !m_Pause)
            {
                m_Timer.Stop();
                if (m_TetrisCore.Descend() == false)
                {
                    m_Jeu = false;
                    if (TetrisPerdu != null)
                    {
                        TetrisPerduEventArgs tpeva = new TetrisPerduEventArgs(m_TetrisCore.Score);
                        TetrisPerdu(this, tpeva);
                    }/*
                    if (m_TetrisInfos != null)
                    {
                        m_TetrisInfos.Perdu(m_TetrisCore.Score);
                    }*/
                }
                else
                {
                    m_Timer.Interval = m_TetrisCore.Delai;
                    m_Timer.Start();
                }
            }
        }

        /// <summary>
        /// Initialise l'objet afin de lancer une nouvelle partie
        /// </summary>
        public void Play()
        {
            m_Jeu = true; 
            m_Pause = false;
            m_TetrisCore = new TetrisCore(this);
            m_Timer.Interval = m_TetrisCore.Delai;
            m_Timer.Start();
        }

        /// <summary>
        /// Pause
        /// </summary>
        public void Pause()
        {
            m_Pause = !m_Pause;
        }
        /*
        public ITetrisInfos TetrisInfos
        {
            set { m_TetrisInfos = value; }
        }
         */

    }
}
